package com.werken.classworlds;

/*
 $Id: Configurator.java,v 1.3 2002/09/24 20:18:19 bob Exp $

 Copyright 2002 (C) The Werken Company. All Rights Reserved.
 
 Redistribution and use of this software and associated documentation
 ("Software"), with or without modification, are permitted provided
 that the following conditions are met:

 1. Redistributions of source code must retain copyright
    statements and notices.  Redistributions must also contain a
    copy of this document.
 
 2. Redistributions in binary form must reproduce the
    above copyright notice, this list of conditions and the
    following disclaimer in the documentation and/or other
    materials provided with the distribution.
 
 3. The name "classworlds" must not be used to endorse or promote
    products derived from this Software without prior written
    permission of The Werken Company.  For written permission,
    please contact bob@werken.com.
 
 4. Products derived from this Software may not be called "classworlds"
    nor may "classworlds" appear in their names without prior written
    permission of The Werken Company. "classworlds" is a registered
    trademark of The Werken Company.
 
 5. Due credit should be given to The Werken Company.
    (http://classworlds.werken.com/).
 
 THIS SOFTWARE IS PROVIDED BY THE WERKEN COMPANY AND CONTRIBUTORS
 ``AS IS'' AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT
 NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL
 THE WERKEN COMPANY OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 OF THE POSSIBILITY OF SUCH DAMAGE.
 
 */

import java.io.File;
import java.io.FileReader;
import java.io.BufferedReader;
import java.io.IOException;
import java.net.MalformedURLException;

/** <code>Launcher</code> configurator.
 *
 *  @author <a href="mailto:bob@eng.werken.com">bob mcwhirter</a>
 *
 *  @version $Id: Configurator.java,v 1.3 2002/09/24 20:18:19 bob Exp $
 */
class Configurator
{
    // ------------------------------------------------------------
    //     Constants
    // ------------------------------------------------------------

    /** Main spec prefix. */
    public static final String MAIN_PREFIX = "main is";

    /** Import spec prefix. */
    public static final String IMPORT_PREFIX = "import";

    /** Load spec prefix. */
    public static final String LOAD_PREFIX = "load";

    // ------------------------------------------------------------
    //     Instance members
    // ------------------------------------------------------------

    /** The launcher to configure. */
    private Launcher launcher;

    // ------------------------------------------------------------
    //     Constructors
    // ------------------------------------------------------------

    /** Construct.
     *
     *  @param launcher The launcher to configure.
     */
    Configurator(Launcher launcher)
    {
        this.launcher = launcher;
    }

    // ------------------------------------------------------------
    //     Instance methods
    // ------------------------------------------------------------

    /** Configure from a file.
     *
     *  @param confPath Path to the config file.
     *
     *  @throws IOException If an error occurs reading the config file.
     *  @throws MalformedURLException If the config file contains invalid URLs.
     *  @throws ConfigurationException If the config file is corrupt.
     *  @throws DuplicateRealmException If the config file defines two realms with the same id.
     *  @throws NoSuchRealmException If the config file defines a main entry point in
     *          a non-existent realm.
     */
    void configure(String confPath)
        throws IOException, MalformedURLException, ConfigurationException, DuplicateRealmException, NoSuchRealmException
    {
        File confFile = new File( confPath );

        FileReader fileReader = new FileReader( confFile );

        BufferedReader reader = new BufferedReader( fileReader );

        ClassWorld world    = new ClassWorld();
        ClassRealm curRealm = null;

        String line = null;

        int lineNo = 0;
        boolean mainSet = false;

        while ( true )
        {
            line = reader.readLine();

            if ( line == null )
            {
                break;
            }

            ++lineNo;
            line = line.trim();

            if ( canIgnore( line ) )
            {
                continue;
            }

            if ( line.startsWith( MAIN_PREFIX ) )
            {
                if ( mainSet )
                {
                    throw new ConfigurationException( "Duplicate main configuration (" + lineNo + "): " + line );
                }

                String conf = line.substring( MAIN_PREFIX.length() ).trim();

                int fromLoc = conf.indexOf( "from" );

                String mainClassName = conf.substring( 0,
                                                       fromLoc ).trim();

                String mainRealmName = conf.substring( fromLoc + 4 ).trim();
                
                this.launcher.setAppMain( mainClassName,
                                          mainRealmName );

                mainSet = true;
            }
            else if ( line.startsWith( "[" ) )
            {
                int rbrack = line.indexOf( "]" );

                if ( rbrack < 0 )
                {
                    throw new ConfigurationException( "Invalid realm specifier (" + lineNo + "): " + line );
                }

                String realmName = line.substring( 1,
                                                   rbrack );

                curRealm = world.newRealm( realmName );
            }
            else if ( line.startsWith( IMPORT_PREFIX ) )
            {
                if ( curRealm == null )
                {
                    throw new ConfigurationException( "Unhandled import (" + lineNo +"): " + line );
                }

                String conf = line.substring( IMPORT_PREFIX.length() ).trim();

                int fromLoc = conf.indexOf( "from" );

                String importSpec = conf.substring( 0,
                                                    fromLoc ).trim();

                String relamName = conf.substring( fromLoc + 4 ).trim();

                curRealm.importFrom( relamName,
                                     importSpec );

            }
            else if ( line.startsWith( LOAD_PREFIX ) )
            {
                String constituent = line.substring( LOAD_PREFIX.length() ).trim();

                constituent = filter( constituent );

                curRealm.addConstituent( new File( constituent ).toURL() );
            }
            else
            {
                throw new ConfigurationException( "Unhandled configuration (" + lineNo +"): " + line );
            }
        }

        this.launcher.setWorld( world );

        reader.close();
    }

    /** Filter a string for system properties.
     *
     *  @param text The text to filter.
     *
     *  @return The filtered text.
     *
     *  @throws ConfigurationException If the property does not
     *          exist or if there is a syntax error.
     */
    protected String filter(String text) throws ConfigurationException
    {
        String result = "";

        int cur = 0;
        int textLen = text.length();

        int propStart = -1;
        int propStop  = -1;

        String propName = null;
        String propValue = null;

        while ( cur < textLen )
        {
            propStart = text.indexOf( "${",
                                      cur );

            if ( propStart < 0 )
            {
                break;
            }

            result += text.substring(cur,
                                     propStart );

            propStop = text.indexOf( "}",
                                     propStart );

            if ( propStop < 0 )
            {
                throw new ConfigurationException( "Unterminated property: " + text.substring( propStart ) );
            }

            propName = text.substring( propStart + 2,
                                       propStop );

            propValue = System.getProperty( propName );

            if ( propValue == null )
            {
                throw new ConfigurationException( "No such property: " + propName );
            }
            result += propValue;

            cur = propStop + 1;
        }

        result += text.substring( cur );

        return result;
    }

    /** Determine if a line can be ignored because it is
     *  a comment or simply blank.
     *
     *  @param line The line to test.
     *
     *  @return <code>true</code> if the line is ignorable,
     *          otherwise <code>false</code>.
     */
    private boolean canIgnore(String line)
    {
        return ( line.length() == 0
                 ||
                 line.startsWith( "#" ) );
    }
}

