package com.werken.classworlds;

import junit.framework.TestCase;

import java.io.File;
import java.io.FileNotFoundException;
import java.util.Collection;

public class ConfiguratorTest extends TestCase
{
    private Launcher launcher;
    private Configurator configurator;

    public ConfiguratorTest(String name)
    {
        super( name );
    }

    public void setUp()
    {
        this.launcher = new Launcher();
        this.configurator = new Configurator( this.launcher );
    }

    public void tearDown()
    {
        this.launcher = null;
        this.configurator = null;
    }

    public void testConfigure_Nonexistant() throws Exception
    {
        try
        {
            this.configurator.configure( getConfigPath( "notfound.conf" ) );
            fail( "throw FileNotFoundException" );
        }
        catch (FileNotFoundException e)
        {
            // expected and correct
        }
    }

    public void testConfigure_DuplicateMain() throws Exception
    {
        try
        {
            this.configurator.configure( getConfigPath( "dupe-main.conf" ) );
            fail( "throw ConfigurationException" );
        }
        catch (ConfigurationException e)
        {
            // expected and correct
            assertTrue( e.getMessage().startsWith( "Duplicate main" ) );
        }
    }

    public void testConfigure_DuplicateRealm() throws Exception
    {
        try
        {
            this.configurator.configure( getConfigPath( "dupe-realm.conf" ) );
            fail( "throw DuplicateRealmException" );
        }
        catch (DuplicateRealmException e)
        {
            // expected and correct
            assertEquals( "dupe.realm",
                          e.getId() );
        }
    }

    public void testConfigure_EarlyImport() throws Exception
    {
        try
        {
            this.configurator.configure( getConfigPath( "early-import.conf" ) );
            fail( "throw ConfigurationException" );
        }
        catch (ConfigurationException e)
        {
            // expected and correct
            assertTrue( e.getMessage().startsWith( "Unhandled import" ) );
        }
    }

    public void testConfigure_RealmSyntax() throws Exception
    {
        try
        {
            this.configurator.configure( getConfigPath( "realm-syntax.conf" ) );
            fail( "throw ConfigurationException" );
        }
        catch (ConfigurationException e)
        {
            // expected and correct
            assertTrue( e.getMessage().startsWith( "Invalid realm" ) );
        }
    }

    public void testConfigure_Valid() throws Exception
    {
        this.configurator.configure( getConfigPath( "valid.conf" ) );

        assertEquals( "org.apache.maven.app.App",
                      this.launcher.getMainClassName() );

        assertEquals( "maven",
                      this.launcher.getMainRealmName() );

        ClassWorld world = this.launcher.getWorld();

        Collection realms = world.getRealms();

        assertEquals( 3,
                      realms.size() );

        assertNotNull( world.getRealm( "ant" ) );
        assertNotNull( world.getRealm( "maven" ) );
        assertNotNull( world.getRealm( "xml" ) );

        ClassRealmImpl antRealm = world.getRealmImpl( "ant" );
        ClassRealmImpl mavenRealm = world.getRealmImpl( "maven" );
        ClassRealmImpl xmlRealm = world.getRealmImpl( "xml" );

        assertSame( antRealm,
                    antRealm.locateSourceRealm( "org.apache.tools.Ant" ) );

        assertSame( xmlRealm,
                    antRealm.locateSourceRealm( "org.xml.sax.SAXException" ) );

        assertSame( mavenRealm,
                    mavenRealm.locateSourceRealm( "org.apache.maven.app.App" ) );

        assertSame( xmlRealm,
                    mavenRealm.locateSourceRealm( "org.xml.sax.SAXException" ) );
    }

    public void testConfigure_Unhandled() throws Exception
    {
        try
        {
            this.configurator.configure( getConfigPath( "unhandled.conf" ) );
            fail( "throw ConfigurationException" );
        }
        catch (ConfigurationException e)
        {
            // expected and correct
            assertTrue( e.getMessage().startsWith( "Unhandled configuration" ) );
        }
    }

    public void testFilter_Unterminated() throws Exception
    {
        try
        {
            this.configurator.filter( "${cheese" );
            fail( "throw ConfigurationException" );
        }
        catch (ConfigurationException e)
        {
            // expected and correct
            assertTrue( e.getMessage().startsWith( "Unterminated" ) );
        }
    }

    public void testFilter_Solitary() throws Exception
    {
        System.setProperty( "classworlds.test.prop",
                            "test prop value" );

        String result = this.configurator.filter( "${classworlds.test.prop}" );

        assertEquals( "test prop value",
                      result );
    }

    public void testFilter_AtStart() throws Exception
    {
        System.setProperty( "classworlds.test.prop",
                            "test prop value" );

        String result = this.configurator.filter( "${classworlds.test.prop}cheese" );

        assertEquals( "test prop valuecheese",
                      result );
    }

    public void testFilter_AtEnd() throws Exception
    {
        System.setProperty( "classworlds.test.prop",
                            "test prop value" );

        String result = this.configurator.filter( "cheese${classworlds.test.prop}" );

        assertEquals( "cheesetest prop value",
                      result );
    }

    public void testFilter_Multiple() throws Exception
    {
        System.setProperty( "classworlds.test.prop.one",
                            "test prop value one" );

        System.setProperty( "classworlds.test.prop.two",
                            "test prop value two" );

        String result = this.configurator.filter( "I like ${classworlds.test.prop.one} and ${classworlds.test.prop.two} a lot" );

        assertEquals( "I like test prop value one and test prop value two a lot",
                      result );
    }

    public void testFilter_NonExistent() throws Exception
    {
        try
        {
            String result = this.configurator.filter( "${gollygeewillikers}" );
            fail( "throw ConfigurationException" );
        }
        catch (ConfigurationException e)
        {
            // expected and correct
            assertTrue( e.getMessage().startsWith( "No such property" ) );
        }
    }

    public void testFilter_InMiddle() throws Exception
    {
        System.setProperty( "classworlds.test.prop",
                            "test prop value" );

        String result = this.configurator.filter( "cheese${classworlds.test.prop}toast" );

        assertEquals( "cheesetest prop valuetoast",
                      result );
    }

    private String getConfigPath(String name)
    {
        return new File( new File( System.getProperty( "basedir" ),
                                   "test-data" ),
                         name ).getPath();
    }
}
